﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Markup;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls;
using System.Collections;
using Telerik.Windows;

namespace TreeViewExpandByPath
{
	public partial class Page : UserControl
	{
		static Page()
		{
			EventManager.RegisterClassHandler(typeof(RadTreeViewItem),
				RadContextMenu.OpenedEvent,
				new RoutedEventHandler(OnContextMenuOpenAtTreeViewItem));

			EventManager.RegisterClassHandler(typeof(RadTreeViewItem),
				RadMenuItem.ClickEvent,
				new RoutedEventHandler(OnContextMenuClick));
		}

		private static void OnContextMenuOpenAtTreeViewItem(object sender, RoutedEventArgs args)
		{
			//Cast to RadRoutedEventArgs which have an OriginalSource, this is the ContextMenu:   
			var radArgs = args as RadRoutedEventArgs;

			//the sender is the TreeViewItem:   
			var treeViewItem = sender as RadTreeViewItem;

			//Do stuff:   

			//Make sure you handle the event:   
			radArgs.Handled = true;
		}

		private static void OnContextMenuClick(object sender, RoutedEventArgs args)
		{
			//Cast to RadRoutedEventArgs which have an OriginalSource, this is the ContextMenu:   
			var radArgs = args as RadRoutedEventArgs;

			//the sender is the TreeViewItem:   
			var treeViewItem = sender as RadTreeViewItem;

			//Do stuff:   

			//Make sure you handle the event:   
			radArgs.Handled = true;

		}  


		public static OrganisationModel GetTopOrganisation()
		{
			return GetTopOrganisation("");
		}

		public static OrganisationModel GetTopOrganisation(string name)
		{
			return new OrganisationModel()
			{
				OrganisationName = String.Format("Top Organisation {0}", name),
				Children = new ObservableCollection<object>()
				{ 
					new PersonModel(){ Name = String.Format("Person1 {0}",name) },
					new OrganisationModel()
					{ 
						OrganisationName = String.Format("Organisation 1.1 {0}",name),
						Children = new ObservableCollection<object>()
						{
							new OrganisationModel()
							{
								OrganisationName = String.Format("Organisation 2.1 {0}",name),
								Children = new ObservableCollection<object>()
								{
									new OrganisationModel()
									{
										OrganisationName = String.Format("Organisation 3.1 {0}",name),
										Children = new List<object>(){
											new PersonModel()
											{
												Name = "Person 3.1.1"
											}
										}
									}
								}
							},
							new PersonModel(){ Name = String.Format("Person 1.1 {0}",name) },
							new PersonModel(){ Name = String.Format("Person 1.2 {0}",name) }
						}
					}
				}
			};
		}

		public Page()
		{
			InitializeComponent();

			treeView.ItemsSource = new object[] { GetTopOrganisation("A") };

			//This will not work:
			//ExpandItemAndSelect(); 

			//But this will work:
			this.Dispatcher.BeginInvoke(ExpandItemAndSelect);
			//The above is the same as this:
			//this.Dispatcher.BeginInvoke(() => ExpandItemAndSelect());

		}

		private void ExpandItemAndSelect()
		{
			var path = "Top Organisation A|Organisation 1.1 A|Organisation 2.1 A|Organisation 3.1 A|Person 3.1.1";
			treeView.ExpandItemByPath(path, "|");
			treeView.GetItemByPath(path, "|").IsSelected = true;
		}

		private void Button_Click(object sender, RoutedEventArgs e)
		{
			treeView.SelectedItems.Clear();
		}

		private void TextBox_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.Enter)
			{
				//Do Search.
				//e.Handled = true;
			}
		}
	}

	[ContentProperty("TypedTemplates")]
	public class TypeTemplateSelector : DataTemplateSelector
	{
		public TypeTemplateSelector()
		{
			TypedTemplates = new List<TypedTemplate>();
		}

		public override DataTemplate SelectTemplate(object item, DependencyObject container)
		{
			var typeName = item.GetType().Name;

			foreach (var contentItem in TypedTemplates)
			{
				var typedTemplate = contentItem as TypedTemplate;
				if (typedTemplate.TypeName == typeName)
				{
					return typedTemplate.DataTemplate;
				}
			}
			return null;
		}

		public IList TypedTemplates
		{
			private set;
			get;
		}
	}

	[ContentProperty("DataTemplate")]
	public class TypedTemplate
	{
		public String TypeName { get; set; }
		public DataTemplate DataTemplate { get; set; }
	}

	public class PersonModel
	{
		public String Name { get; set; }

		public override string ToString()
		{
			return Name;
		}
	}

	public class OrganisationModel
	{
		public OrganisationModel()
		{
			Children = new ObservableCollection<Object>();
		}
		public String OrganisationName { get; set; }
		public IList<Object> Children { get; set; }

		public override string ToString()
		{
			return OrganisationName;
		}
	}

	public class Wow : RadExpander
	{
		protected override void OnKeyDown(KeyEventArgs e)
		{
			base.OnKeyDown(e);
		}
	}
}
